@@ -10,7 +10,7 @@ for the Microsoft Visual Studio.
 
 # This module should be kept compatible with Python 2.1.
 
-__revision__ = "$Id: msvccompiler.py,v 1.64 2004/11/28 01:10:01 tim_one Exp $"
+__revision__ = "$Id: msvccompiler.py,v 1.64.2.3 2005/03/11 17:20:41 tim_one Exp $"
 
 import sys, os, string
 from distutils.errors import \
@@ -211,6 +211,9 @@ class MSVCCompiler (CCompiler) :
             self.__macros = MacroExpander(self.__version)
         else:
             self.__root = r"Software\Microsoft\Devstudio"
+        self.initialized = False
+
+    def initialize(self):
         self.__paths = self.get_msvc_paths("path")
 
         if len (self.__paths) == 0:
@@ -252,6 +255,8 @@ class MSVCCompiler (CCompiler) :
                 ]
         self.ldflags_static = [ '/nologo']
 
+        self.initialized = True
+
 
     # -- Worker methods ------------------------------------------------
 
@@ -290,6 +295,7 @@ class MSVCCompiler (CCompiler) :
                 output_dir=None, macros=None, include_dirs=None, debug=0,
                 extra_preargs=None, extra_postargs=None, depends=None):
 
+        if not self.initialized: self.initialize()
         macros, objects, extra_postargs, pp_opts, build = \
                 self._setup_compile(output_dir, macros, include_dirs, sources,
                                     depends, extra_postargs)
@@ -381,6 +387,7 @@ class MSVCCompiler (CCompiler) :
                            debug=0,
                            target_lang=None):
 
+        if not self.initialized: self.initialize()
         (objects, output_dir) = self._fix_object_args (objects, output_dir)
         output_filename = \
             self.library_filename (output_libname, output_dir=output_dir)
@@ -414,6 +421,7 @@ class MSVCCompiler (CCompiler) :
               build_temp=None,
               target_lang=None):
 
+        if not self.initialized: self.initialize()
         (objects, output_dir) = self._fix_object_args (objects, output_dir)
         (libraries, library_dirs, runtime_library_dirs) = \
             self._fix_lib_args (libraries, library_dirs, runtime_library_dirs)
