@@ -28,7 +28,7 @@ import shutil
 import time
 
 __all__ = ["PimpPreferences", "PimpDatabase", "PimpPackage", "main",
-    "PIMP_VERSION", "main"]
+    "getDefaultDatabase", "PIMP_VERSION", "main"]
 
 _scriptExc_NotInstalled = "pimp._scriptExc_NotInstalled"
 _scriptExc_OldInstalled = "pimp._scriptExc_OldInstalled"
@@ -36,12 +36,13 @@ _scriptExc_BadInstalled = "pimp._scriptExc_BadInstalled"
 
 NO_EXECUTE=0
 
-PIMP_VERSION="0.4"
+PIMP_VERSION="0.5"
 
 # Flavors:
 # source: setup-based package
 # binary: tar (or other) archive created with setup.py bdist.
-DEFAULT_FLAVORORDER=['source', 'binary']
+# installer: something that can be opened
+DEFAULT_FLAVORORDER=['source', 'binary', 'installer']
 DEFAULT_DOWNLOADDIR='/tmp'
 DEFAULT_BUILDDIR='/tmp'
 DEFAULT_INSTALLDIR=distutils.sysconfig.get_python_lib()
@@ -55,7 +56,7 @@ def getDefaultDatabase(experimental=False):
 
     major, minor, micro, state, extra = sys.version_info
     pyvers = '%d.%d' % (major, minor)
-    if state != 'final':
+    if micro == 0 and state != 'final':
         pyvers = pyvers + '%s%d' % (state, extra)
 
     longplatform = distutils.util.get_platform()
@@ -401,23 +402,30 @@ class PimpDatabase:
             self._maintainer = plistdata.get('Maintainer', '')
             self._description = plistdata.get('Description', '').strip()
             self._url = url
-        self._appendPackages(plistdata['Packages'])
+        self._appendPackages(plistdata['Packages'], url)
         others = plistdata.get('Include', [])
-        for url in others:
-            self.appendURL(url, included=1)
+        for o in others:
+            o = urllib.basejoin(url, o)
+            self.appendURL(o, included=1)
 
-    def _appendPackages(self, packages):
+    def _appendPackages(self, packages, url):
         """Given a list of dictionaries containing package
         descriptions create the PimpPackage objects and append them
         to our internal storage."""
 
         for p in packages:
             p = dict(p)
+            if p.has_key('Download-URL'):
+                p['Download-URL'] = urllib.basejoin(url, p['Download-URL'])
             flavor = p.get('Flavor')
             if flavor == 'source':
                 pkg = PimpPackage_source(self, p)
             elif flavor == 'binary':
                 pkg = PimpPackage_binary(self, p)
+            elif flavor == 'installer':
+                pkg = PimpPackage_installer(self, p)
+            elif flavor == 'hidden':
+                pkg = PimpPackage_installer(self, p)
             else:
                 pkg = PimpPackage(self, dict(p))
             self._packages.append(pkg)
@@ -543,7 +551,7 @@ class PimpPackage:
             rv = rv + '-%s' % self._dict['Version']
         if self._dict.has_key('Flavor'):
             rv = rv + '-%s' % self._dict['Flavor']
-        if not self._dict.get('Download-URL'):
+        if self._dict.get('Flavor') == 'hidden':
             # Pseudo-package, show in parentheses
             rv = '(%s)' % rv
         return rv
@@ -620,11 +628,11 @@ class PimpPackage:
             if status == "yes":
                 return []
             return [(None,
-                "%s: This package cannot be installed automatically (no Download-URL field)" %
+                "Package %s cannot be installed automatically, see the description" %
                     self.fullname())]
         if self.systemwideOnly() and self._db.preferences.isUserInstall():
             return [(None,
-                "%s: This package can only be installed system-wide" %
+                "Package %s can only be installed system-wide" %
                     self.fullname())]
         if not self._dict.get('Prerequisites'):
             return []
@@ -791,7 +799,7 @@ class PimpPackage_binary(PimpPackage):
             return "%s: Binary package cannot have Install-command" % self.fullname()
 
         if self._dict.has_key('Pre-install-command'):
-            if _cmd(output, self._buildDirname, self._dict['Pre-install-command']):
+            if _cmd(output, '/tmp', self._dict['Pre-install-command']):
                 return "pre-install %s: running \"%s\" failed" % \
                     (self.fullname(), self._dict['Pre-install-command'])
 
@@ -824,7 +832,7 @@ class PimpPackage_binary(PimpPackage):
         self.afterInstall()
 
         if self._dict.has_key('Post-install-command'):
-            if _cmd(output, self._buildDirname, self._dict['Post-install-command']):
+            if _cmd(output, '/tmp', self._dict['Post-install-command']):
                 return "%s: post-install: running \"%s\" failed" % \
                     (self.fullname(), self._dict['Post-install-command'])
 
@@ -891,6 +899,37 @@ class PimpPackage_source(PimpPackage):
                     (self.fullname(), self._dict['Post-install-command'])
         return None
 
+class PimpPackage_installer(PimpPackage):
+
+    def unpackPackageOnly(self, output=None):
+        """We don't unpack dmg packages until installing"""
+        pass
+
+    def installPackageOnly(self, output=None):
+        """Install a single source package.
+        
+        If output is given it should be a file-like object and it
+        will receive a log of what happened."""
+                    
+        if self._dict.has_key('Post-install-command'):
+            return "%s: Installer package cannot have Post-install-command" % self.fullname()
+
+        if self._dict.has_key('Pre-install-command'):
+            if _cmd(output, '/tmp', self._dict['Pre-install-command']):
+                return "pre-install %s: running \"%s\" failed" % \
+                    (self.fullname(), self._dict['Pre-install-command'])
+                    
+        self.beforeInstall()
+
+        installcmd = self._dict.get('Install-command')
+        if installcmd:
+            if '%' in installcmd:
+                installcmd = installcmd % self.archiveFilename
+        else:
+                installcmd = 'open \"%s\"' % self.archiveFilename
+        if _cmd(output, "/tmp", installcmd):
+            return '%s: install command failed (use verbose for details)' % self.fullname()
+        return '%s: downloaded and opened. Install manually and restart Package Manager' % self.archiveFilename
 
 class PimpInstaller:
     """Installer engine: computes dependencies and installs
